/*********************************************************************

 Author        : ADI - Apps            www.analog.com/MicroConverter

 Date          : May 2008

 File          : LIN_FRAME_MASTER_Erase - Protocol 4.c

 Hardware      : Applicable to ADuC7038/9

 Description   : Wake-up Timer causes part to power-up every 8s.
 				 the timer is configured in Hr:Min:Sec:Hundredths format
				
				Additional (Gilbert): Can be woke up by LIN during power down mode.

				 T2LD:
				 bits 31-24 represent the hours
				 bits 21-16 represent the minutes
				 bits 13-8 represent the secondes
				 bits 6-0 represent the hundredths

				 The debugger cannot be used when the part is placed in power down mode.
		
*********************************************************************/

#include <ADuC7038.h>

#define GPIO_0  0x00010000

void LIN_INIT(void);
void My_FIQ_Function(void);
void My_IRQ_Function(void);

void delay(int);
char HVstatus;
int iFIQ_Source,itemp1,itemp2,itemp3, iIRQSIG;
int FRAME_count = 0;
int FRAME_ERROR = 0;

int main()
{
	iFIQ_Source = FIQSTA;			// Save a temporary copy of IRQSTA	
	GPCON = 0x100000;						// enable LIN Rx and LIN Tx on GPIO
	LIN_INIT();
//  GPCON = 0x00000000;	//   all GPIOs
    GPDAT = 0x0F0F0000;	// Set all bits to outputs	- high

	IRQ = My_IRQ_Function; 	// Initialise Interrupt Routine	 for LIN
	IRQEN = 0x08;			// Enable Timer 1 interrupt	  - 32 bits
//	IRQEN = 0x20;			// Enable LIN interrupt
							// Count down, Enable Timer 2, hours mode
	T1CON = 0x28;			// T1 en, Core Clock
	T1LD = 	0x215;			// See timing  - 52.08 uS	   - period
	
	while(1)
								
	{
	}				// Infinite Loop
}
void My_IRQ_Function() 
{
	T1CLRI = 0;		// clear interrupt	
 	if (FRAME_count == 0)
		T1LD = 	0x215;	

	switch(FRAME_count)			  // Protocol 4
	{
	case 0:	 GPCLR = GPIO_0; break;		   // start
	case 1:	 break;
	case 2:	 break;
	case 3:	break;
	case 4:	 break;
	case 5:	 break;
	case 6:	 break;
	case 7:	 break;
	case 8:	 break;					          // Break Field
	case 9:	 /*FRAME_count = 24;*/  break;	
 	case 10: /*FRAME_count = 24;*/	break;	   // only 11 bits
	case 11: /*FRAME_count = 24;*/	break;
	case 12: /*FRAME_count = 24;*/	break;	
	case 13: FRAME_count = 24; 	break;	
	case 14: break;	
	case 15: break;	
	case 16: break;				 // ignore all these
	case 17: break;	
	case 18: break;	
	case 19: break;	
	case 20: break;	
	case 21: break;	
	case 22: break;	
	case 23: break;	
	case 24: GPCLR = GPIO_0; break;		 // Break Field
	case 25: GPSET = GPIO_0; break;		   // stop
	case 26: GPSET = GPIO_0; break;		

	case 27: GPCLR = GPIO_0; break;	   // start SYNC Field	- START bit
	case 28: GPSET = GPIO_0; break;		// 1	
	case 29: GPCLR = GPIO_0; break;	   	// 0
	case 30: GPSET = GPIO_0; break;		// 1	
	case 31: GPCLR = GPIO_0; break;	   	// 0		SYNC field 0x55
	case 32: GPSET = GPIO_0; break;		// 1
	case 33: GPCLR = GPIO_0; break;	   	// 0
	case 34: GPSET = GPIO_0; break;		// 1
	case 35: GPCLR = GPIO_0; break;	   	// 0
	case 36: GPSET = GPIO_0; break;		// STOP    bit

	case 37: GPCLR = GPIO_0; break;	    //  PID Field	- START bit
	case 38: GPCLR = GPIO_0; break;		// 0	
	case 39: GPCLR = GPIO_0; break;	   	// 0
	case 40: GPCLR = GPIO_0; break;		// 0	
	case 41: GPCLR = GPIO_0; break;	   	// 0		PID field 0x30	- Master requests frame from SLAVE
	case 42: GPSET = GPIO_0; break;		// 1
	case 43: GPSET = GPIO_0; break;	   	// 1
	case 44: GPSET = GPIO_0; break;		// 1		// P0	 parity
	case 45: GPSET = GPIO_0; break;	   	// 1		//p1
	case 46: GPSET = GPIO_0; break;		// STOP    bit	  inter byte space

	case 47: GPCLR = GPIO_0; break;	    //  BYTE Field	- START bit	0	  *****************
	case 48: GPCLR = GPIO_0; break;		// 0
	case 49: GPCLR = GPIO_0; break;	   	// 0
	case 50: GPCLR = GPIO_0; break;		// 0	
	case 51: GPCLR = GPIO_0; break;	   	// 0		BYTE 	 0x30
	case 52: GPSET = GPIO_0; break;		// 1
	case 53: GPSET = GPIO_0; break;	   	// 1
	case 54: GPCLR = GPIO_0; break;		// 0		
	case 55: GPCLR = GPIO_0; break;	   	// 0		 
	case 56: GPSET = GPIO_0; break;		// STOP    bit

	case 57: GPCLR = GPIO_0; break;	    //  CHECKSUM Field	- START bit
	case 58: GPCLR = GPIO_0; break;		// 0
	case 59: GPSET = GPIO_0; break;	   	// 1
	case 60: GPSET = GPIO_0; break;		// 1	
	case 61: GPSET = GPIO_0; break;	   	// 1		CHECKSUM 	 = 0xDE - see Rough Work
	case 62: GPSET = GPIO_0; break;		// 1
	case 63: GPCLR = GPIO_0; break;	   	// 0
	case 64: GPSET = GPIO_0; break;		// 1		
	case 65: GPSET = GPIO_0; break;	   	// 1		 
	case 66: GPSET = GPIO_0; break;		// STOP    bit		
	default: 		// End of Frame 
	 	break;
	}
	 FRAME_count++;
   if (FRAME_count > 67)
   	{
	T1LD = 	0x01;	
   	 FRAME_count = 0;	// 99 means No repeat
//	 delay(90);
	 }
//   	delay(30);
		
	return ;
}


void delay (int length)
{
	while (length >=0)
    	length--;
}

void LIN_INIT(void )
{
//	GPCON = 0x110000;						// enable LIN Rx and LIN Tx on GPIO
  	LINCON = 0x1A00;							// Enable LIN
	do										
	{
		HVDAT = 0x01; 						// Enable Normal LIN TX mode / reenable short circuit
		HVCON = 0x08; 						// Write to Config
		do
		{
			HVstatus = HVCON;
		}
		while(HVstatus & 0x1); 				//  command finished
	}
	while (!(HVstatus & 0x4));				// transmit command 

// LINWU = 0x0010;
// LINBK = 0x030;							// Set number of 131kHz periods to generate a Break Interrupt
											// 0x3F / 131kHz ~ 480us which is just over 9.5 TBits.
											// According to LIN V2.0, a slave with a clock accuracy of <2% may
											// use a break detection of 9.5 bit, otherwise a break detection of
											// 11 bits is required
											// This is for 19200bps
}

